#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <DNSServer.h>
#include <ESP8266mDNS.h>
#include <EEPROM.h>
#include <ESP8266HTTPClient.h>
#include <I2Cdev.h>
#include <MPU6050.h>
#include <Wire.h>

#define DEBUG
#include "debug.h"
#include "params.h"
#include "tools.h"
#include "eeprom.h"
#include "network.h"
#include "http.h"
#include "acc.h"
#include "tonegen.h"

#define STATE_UNINITIALIZED 100
#define STATE_AWAITING_WIFI_CONFIGURATION 200
#define STATE_LAUNCHING_WIFI_CONFIGURATION 300
#define STATE_TESTING_WIFI_CONFIGURATION 400
#define STATE_REGISTRATION 500
#define STATE_COUNTING_TURNS 600
#define STATE_REPORTING 700

int state;
int turns_counted;

void setup() {
  //pinMode(RESET_ENABLE_PIN, WAKEUP_PULLUP);
  pinMode(RESET_ENABLE_PIN, OUTPUT);
  digitalWrite(RESET_ENABLE_PIN, LOW);
  pinMode(GY521_ENABLE_PIN, OUTPUT);
  digitalWrite(GY521_ENABLE_PIN, HIGH);
  DEBUG_BEGIN();
  DEBUG_PRINTLN("Starting");  
  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, LOW);
  delay(100);
  digitalWrite(LED_BUILTIN, HIGH);
  acc_setup();
  eeprom_setup();
  network_setup();

  DEBUG_PRINT("credentials found? "); DEBUG_PRINTLN(eeprom_checkCredentialsExist() ? "YES" : "NO");
  DEBUG_PRINT("credentials valid? ");
  switch (eeprom_getCredentialsValidity()) {
    case CREDENTIALS_VALIDITY_FAIL: DEBUG_PRINTLN("NO"); break;
    case CREDENTIALS_VALIDITY_PASS: DEBUG_PRINTLN("YES"); break;
    case CREDENTIALS_VALIDITY_UNKNOWN: DEBUG_PRINTLN("UNKNOWN"); break;
  }

  if (eeprom_checkCredentialsExist() && (eeprom_getCredentialsValidity() == CREDENTIALS_VALIDITY_PASS) &&
      (eeprom_getRegistrationStatus() == REG_STATUS_PASSED)) {
    // creds already loaded from eeprom in eeprom_setup();
    setState(STATE_COUNTING_TURNS);
  }
  else {
    eeprom_clear();
    setState(STATE_UNINITIALIZED);
  }
}

int registrationCounter = 0;
long wifi_setup_tstamp = 0;

void loop() {
  switch (state) {

    case STATE_UNINITIALIZED:
      DEBUG_PRINTLN("Firing up AP");
      network_fireUpAP();
      delay(1000);
      wifi_setup_tstamp = millis();
      setState(STATE_AWAITING_WIFI_CONFIGURATION);
      break;

    case STATE_AWAITING_WIFI_CONFIGURATION:
      network_handleServers();
      if (eeprom_checkCredentialsExist()) {
        setState(STATE_LAUNCHING_WIFI_CONFIGURATION);
      }
      if ((millis() - wifi_setup_tstamp) > WIFI_CONFIGURATION_IDLE_TIMEOUT) {
        sleep();
      }
      break;

    case STATE_LAUNCHING_WIFI_CONFIGURATION:
      DEBUG_PRINTLN("Launching wifi configuration");
      network_connectWifi();
//      eeprom_setCredentialsValidity(CREDENTIALS_VALIDITY_UNKNOWN);
      setState(STATE_TESTING_WIFI_CONFIGURATION);
      break;

    case STATE_TESTING_WIFI_CONFIGURATION:
      DEBUG_PRINTLN("Testing wifi configuration");
      if(network_updateWifiStatus()) {
        if (eeprom_getCredentialsValidity() == CREDENTIALS_VALIDITY_FAIL) {
          play_wifi_failure();
          DEBUG_PRINTLN("play_wifi_failure");
          eeprom_clear();
          wifi_setup_tstamp = millis();
          //setState(STATE_AWAITING_WIFI_CONFIGURATION);
          // need to ofire up AP
          setState(STATE_UNINITIALIZED);
        }
        else { // CREDENTIALS_VALIDITY_PASS or CREDENTIALS_VALIDITY_UNKNOWN, in case of wifi going down
          registrationCounter = REGISTRATION_RETRY_COUNT;
          if (eeprom_getRegistrationStatus() == REG_STATUS_PASSED) {
            setState(STATE_REPORTING);
          }
          else {
            setState(STATE_REGISTRATION);
          }
        }
      }
      break;

    case STATE_REGISTRATION:
      sprintf(registration_url, REGISTRATION_URL, MAC_char, full_name, phone_number);
      if (curl(registration_url) == HTTP_CODE_OK) {
        eeprom_setRegistrationStatus(REG_STATUS_PASSED);
        play_wifi_success();
        DEBUG_PRINTLN("play_wifi_success();");
//        setState(STATE_COUNTING_TURNS);
        sleep();
      }
      else {
        if ((--registrationCounter) == 0) {
          eeprom_setRegistrationStatus(REG_STATUS_FAILED);
          DEBUG_PRINT("registration failed ");
          DEBUG_PRINT(REGISTRATION_RETRY_COUNT);
          DEBUG_PRINTLN(" times, aborting process");
          play_wifi_failure();
          eeprom_clear();
          wifi_setup_tstamp = millis();
          setState(STATE_AWAITING_WIFI_CONFIGURATION);
        }
      }
      break;

    case STATE_COUNTING_TURNS:
    {
      // this will block until accelerometer reports no movement
      int stability = acc_get_stability();
      switch(stability) {
        case ACC_SHAKE:
          play_triple_beep();
          eeprom_setNewRollFlag(true);
          delay(2000); // allow the user to stop shaking the device
          sleep();
        break;
        case ACC_STABLE:
          turns_counted = acc_get_turn_counter();
          acc_reset_turn_counter();
          DEBUG_PRINT("rounds: "); DEBUG_PRINTLN(turns_counted);
          if(network_isConnected()) {
            setState(STATE_REPORTING);
          }
          else {
            setState(STATE_LAUNCHING_WIFI_CONFIGURATION);
          }
        break;
        case ACC_UNSTABLE:
          acc_count_turns();
        break;
      }
      break;
    }
    case STATE_REPORTING:
      {
        turns_counted += ROLE_OFFSET;
        eeprom_storeReport(turns_counted);
        int newRoll = eeprom_getNewRollFlag() ? 1 : 0;
        int* his = eeprom_getStoredReports();
        sprintf(report_url, REPORT_URL, MAC_char, his[0],his[1],his[2],his[3],his[4],his[5],newRoll);
        int retries = REPORT_RETRY_COUNT;
        while(retries-- > 0 && curl(report_url) != HTTP_CODE_OK) {
          delay(500);
        }
        if(retries == 0) {
          DEBUG_PRINTLN("reporting failed... will try again next wakekup");
        }
        else {
          // clear new roll flag from eeprom
          eeprom_setNewRollFlag(false);
        }
        sleep();
      }
      break;
  }
  //if(state == STATE_AWAITING_WIFI_CONFIGURATION ||
}

void setState(int newState) {
#ifdef DEBUG
  switch (newState) {
    case STATE_UNINITIALIZED:
      DEBUG_PRINTLN("Setting state to: STATE_UNINITIALIZED");
      break;
    case STATE_AWAITING_WIFI_CONFIGURATION:
      DEBUG_PRINTLN("Setting state to: STATE_AWAITING_WIFI_CONFIGURATION");
      break;
    case STATE_LAUNCHING_WIFI_CONFIGURATION:
      DEBUG_PRINTLN("Setting state to: STATE_LAUNCHING_WIFI_CONFIGURATION");
      break;
    case STATE_TESTING_WIFI_CONFIGURATION:
      DEBUG_PRINTLN("Setting state to: STATE_TESTING_WIFI_CONFIGURATION");
      break;
    case STATE_REGISTRATION:
      DEBUG_PRINTLN("Setting state to: STATE_REGISTRATION");
      break;
    case STATE_COUNTING_TURNS:
      DEBUG_PRINTLN("Setting state to: STATE_COUNTING_TURNS");
      break;
    case STATE_REPORTING:
      DEBUG_PRINTLN("Setting state to: STATE_REPORTING");
      break;
    default:
      DEBUG_PRINT("Setting state to: UNKNOWN STATE(");
      DEBUG_PRINT(newState);
      DEBUG_PRINTLN(")");
  }
#endif
  state = newState;
}

void sleep() {
    DEBUG_PRINTLN("going to sleep...");
//    ESP.deepSleep(0, WAKE_RF_DISABLED);
    ESP.deepSleep(0, WAKE_RF_DEFAULT);
}

