#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <DNSServer.h>
#include <ESP8266mDNS.h>
#include <EEPROM.h>


#define RESET_CREDENTIALS_PIN D6
#define USE_SERIAL Serial


/* Set these to your desired softAP credentials. They are not configurable at runtime */
const char *softAP_ssid = "SmartWipeAP";
const char *softAP_password = "12345678";

/* hostname for mDNS. Should work at least on windows. Try http://esp8266.local */
const char *myHostname = "esp8266";

/* Don't set this wifi credentials. They are configurated at runtime and stored on EEPROM */
char ssid[32] = "";
char password[32] = "";
uint8_t MAC_array[6];
char MAC_char[18];

// DNS server
const byte DNS_PORT = 53;
DNSServer dnsServer;

// Web server
ESP8266WebServer server(80);

/* Soft AP network parameters */
IPAddress apIP(192, 168, 4, 1);
IPAddress netMsk(255, 255, 255, 0);


/** Should I connect to WLAN asap? */
boolean connect;

/** Last time I tried to connect to WLAN */
long lastConnectTry = 0;

/** Current WLAN status */
int status = WL_IDLE_STATUS;


void setup() {
  delay(1000);
  Serial.begin(9600);
  Serial.println();
  Serial.println("Obtaining MAC address...");
  WiFi.macAddress(MAC_array);
  for (int i = 0; i < sizeof(MAC_array); ++i) {
    sprintf(MAC_char, "%s%02x:", MAC_char, MAC_array[i]);
  }

  Serial.print("Configuring access point...");
  long start = millis();
  /* You can remove the password parameter if you want the AP to be open. */
  WiFi.softAPConfig(apIP, apIP, netMsk);
  WiFi.softAP(softAP_ssid, softAP_password);
  delay(500); // Without delay I've seen the IP address blank
  Serial.print("AP IP address: ");
  Serial.println(WiFi.softAPIP());

  /* Setup the DNS server redirecting all the domains to the apIP */
  dnsServer.setErrorReplyCode(DNSReplyCode::NoError);
  dnsServer.start(DNS_PORT, "*", apIP);

  /* Setup web pages: root, wifi config pages, SO captive portal detectors and not found. */
  server.on("/", handleRoot);
  server.on("/wifi", handleWifi);
  server.on("/wifisave", handleWifiSave);
  server.on("/generate_204", handleRoot);  //Android captive portal. Maybe not needed. Might be handled by notFound handler.
  server.on("/fwlink", handleRoot);  //Microsoft captive portal. Maybe not needed. Might be handled by notFound handler.
  server.onNotFound ( handleNotFound );
  server.begin(); // Web server start
  Serial.print("servers startup time: ");
  Serial.print(millis() - start); Serial.println("ms.");
  pinMode(RESET_CREDENTIALS_PIN, INPUT_PULLUP);
  if (!digitalRead(RESET_CREDENTIALS_PIN)) {
    resetCredentials();
  }
  loadCredentials(); // Load WLAN credentials from EEPROM
  connect = strlen(ssid) > 0; // Request WLAN connect if there is a SSID
}

boolean connectWifi() {
  Serial.println("Connecting as wifi client...");
  WiFi.disconnect();
  delay(1000);
  WiFi.begin ( ssid, password );
  delay(1000);
  int connRes = WiFi.waitForConnectResult();
  Serial.print ( "connRes: " );
  Serial.println ( connRes );
  return connRes;
}

void loop() {
  // first execution takes time
  handleWifi();
  // Do work:
  //Serial.println(millis());

  //DNS
  dnsServer.processNextRequest();
  //HTTP
  server.handleClient();
}

void onConnected() {
  /* Just connected to WLAN */
  Serial.println("");
  Serial.print("Connected to ");
  Serial.println(ssid);
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());

  // Setup MDNS responder
  if (!MDNS.begin(myHostname)) {
    Serial.println("Error setting up MDNS responder!");
  } else {
    Serial.println("mDNS responder started");
    // Add service to MDNS-SD
    MDNS.addService("http", "tcp", 80);
  }
}

/**
 * Connect if needed, otherwise track wireless status and fire 
 * onConnected() and onDisonnected() callbacks
 */
void handleWifi() {
  if (connect) {
    Serial.println ("Connect requested");
    connect = false;
    connectWifi();
    lastConnectTry = millis();
  }
  int s = WiFi.status();
  if (s == 0 && millis() > (lastConnectTry + 60000) ) {
    /* If WLAN disconnected and idle try to connect */
    /* Don't set retry time too low as retry interfere the softAP operation */
    connect = true;
  }
  if (status != s) { // WLAN status change
    Serial.print ("Status: "); Serial.println ( s );
    status = s;
    if (s == WL_CONNECTED) {
      onConnected();
    }
    else if (s == WL_NO_SSID_AVAIL) {
      WiFi.disconnect();
      onDisconnected();
    }
  }
}

/////////////////////////
// MPU 6050
/////////////////////////


