#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <DNSServer.h>
#include <ESP8266mDNS.h>
#include <EEPROM.h>

#include<Wire.h>
#include "I2Cdev.h"
#include "MPU6050_6Axis_MotionApps20.h"

#define INTERRUPT_PIN 14
#define USE_SERIAL Serial

const int MPU_addr = 0x68;                          // I2C address of the MPU-6050

bool dmpReady = false;                  // set true if DMP init was successful
uint8_t mpuIntStatus;                   // holds actual interrupt status byte from MPU
uint8_t devStatus;                      // return status after each device operation (0 = success, !0 = error)
uint16_t packetSize;                    // expected DMP packet size (default is 42 bytes)
uint16_t fifoCount;                     // count of all bytes currently in FIFO
uint8_t fifoBuffer[64];                 // FIFO storage buffer
volatile bool mpuInterrupt = false;     // indicates whether MPU interrupt pin has gone high
Quaternion q;                                           // quaternion container
MPU6050 mpu;

void setup() {
  delay(1000);
  Serial.begin(9600);
  mpu_setup();
}

void loop() {
  mpu_update();
  return;
}

/////////////////////////
// MPU 6050
/////////////////////////

void dmpDataReady()
{
  mpuInterrupt = true;
}

void mpu_setup() {
  //Begin I2C data transfer
  Wire.begin(4, 5);                 // SCL <==> D1 , SDA <==> D2
  Wire.setClock(400000);            // 400kHz I2C clock. Comment this line if having compilation difficulties
  Wire.beginTransmission(MPU_addr);
  Wire.write(0x6B);                 // PWR_MGMT_1 register
  Wire.write(0);                    // set to zero (wakes up the MPU-6050)
  Wire.endTransmission(true);

  //Initialize DMP
  Serial.println(F("Initializing I2C devices..."));
  mpu.initialize();
  pinMode(INTERRUPT_PIN, INPUT);
  // load and configure the DMP
  Serial.println(F("Initializing DMP..."));
  devStatus = mpu.dmpInitialize();

  // verify connection
  Serial.println(F("Testing device connections..."));
  Serial.println(mpu.testConnection() ? F("MPU6050 connection successful") : F("MPU6050 connection failed"));
  delay(500);

  // Gyro offsets.As the sensor bby default will have some offset value
  mpu.setXGyroOffset(220);
  mpu.setYGyroOffset(76);
  mpu.setZGyroOffset(-85);
  mpu.setZAccelOffset(1788);

  if (devStatus == 0)
  {
    // turn on the DMP
    Serial.println(F("Enabling DMP..."));
    mpu.setDMPEnabled(true);

    // enable Arduino interrupt detection
    Serial.println(F("Enabling interrupt detection (Arduino external interrupt 0)..."));
    attachInterrupt(digitalPinToInterrupt(INTERRUPT_PIN), dmpDataReady, RISING);
    mpuIntStatus = mpu.getIntStatus();

    // set DMP Ready flag so the main loop() function can use it
    Serial.println(F("DMP ready! Waiting for first interrupt..."));
    dmpReady = true;

    // get expected DMP packet size for later comparison
    packetSize = mpu.dmpGetFIFOPacketSize();
  }
  else
  {
    // ERROR!
    // 1 = initial memory load failed
    // 2 = DMP configuration updates failed
    // (if it's going to break, usually the code will be 1)
    Serial.print(F("DMP Initialization failed (code "));
    Serial.print(devStatus);
    Serial.println(F(")"));
  }
}


void mpu_update()
{
  // if programming failed, don't try to do anything
  if (!dmpReady) return;

  // reset interrupt flag and get INT_STATUS byte
  mpuInterrupt = false;
  mpuIntStatus = mpu.getIntStatus();

  // get current FIFO count
  fifoCount = mpu.getFIFOCount();

  // check for overflow (indicates code efficiency should be improved to handle the overflow)
  if ((mpuIntStatus & 0x10) || fifoCount == 1024)
  {
    // reset FIFO buffer
    mpu.resetFIFO();
    Serial.println(F("FIFO overflow!"));
  }
  else if (mpuIntStatus & 0x02)
  {
    // wait for correct available data length
    while (fifoCount < packetSize)
    {
      fifoCount = mpu.getFIFOCount();
    }

    // read a packet from FIFO
    mpu.getFIFOBytes(fifoBuffer, packetSize);

    // track FIFO count here in case there is > 1 packet available
    fifoCount -= packetSize;

    //String object to form a concatenated data to be published in the topic "sensorData"
    String data = "";
    mpu.dmpGetQuaternion(&q, fifoBuffer);
    // data = data + String(q.w, 4) + "\t" + String(q.x, 4) + "\t" + String(q.y, 4) + "\t" + String(q.z, 4);
    data = data + String(q.y, 4);
    Serial.println(data);
  }
  else
  {
    //Serial.println("Skipped");
  }
}

